﻿/*===========================================
    Backgrounds for this sample are powered by ThinkGeo Cloud Maps and require
    a Client ID and Secret. These were sent to you via email when you signed up
    with ThinkGeo, or you can register now at https://cloud.thinkgeo.com.
===========================================*/

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Windows;
using ThinkGeo.Core;

namespace DisplayIsoLines
{
    public partial class Sample
    {
        private string wellDepthPointDataFilePath = @"../../Data/PTALResult1.csv";
        private Dictionary<PointShape, double> _wellDepthPointData;
        private GridCell[,] _gridCellMatrix;
        private readonly Stopwatch _stopwatch;
        private readonly string[] _colorStrings = {
            "#333333",
            "#000080",
            "#0000ff",
            "#00ffff",
            "#339a65",
            "#ffff00",
            "#ff9a00",
            "#ff0000",
            "#800000"
        };

        readonly List<double> _isoLineLevels = new List<double>
        {
            0.01     ,
            2.5      ,
            5.00     ,
            10.0     ,
            15.0     ,
            20.0     ,
            25.0     ,
            40.0     ,
            100000.0 ,
        };
        private readonly RectangleShape _extent =
            new PolygonShape("POLYGON((397746 302890,397746 275390,418046 275390,418046 302890,397746 302890))")
                .GetBoundingBox();

        public Sample()
        {
            InitializeComponent();
            _stopwatch = new Stopwatch();
        }

        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {
            btnGenerateGridFile.IsEnabled = true;

            Map1.MapUnit = GeographyUnit.Meter;
            Map1.CurrentExtent = _extent;

            //Load the well depth points and depth data from a text file into the dictionary
            //We cache this at the class level to prevent form loading it multiple times
            _wellDepthPointData = GetWellDepthPointDataFromCSV(wellDepthPointDataFilePath);


            //Add the grid layer, the grid cells, and the well points to the map
            LayerOverlay isoLineOverlay = new LayerOverlay();
            isoLineOverlay.Drawn += isoLineOverlay_Drawn;
            isoLineOverlay.Drawing += isoLineOverlay_Drawing;

            Map1.Overlays.Add("isoLineOverlay", isoLineOverlay);

            isoLineOverlay.Layers.Add("IsoLineLayer", GetGridIsoLineLayer());
            isoLineOverlay.DrawingQuality = DrawingQuality.HighQuality;

            Map1.Refresh();
        }

        private static Dictionary<PointShape, double> GetWellDepthPointDataFromCSV(string csvFilePath)
        {
            StreamReader streamReader = null;
            var wellDataPoints = new Dictionary<PointShape, double>();

            try
            {
                streamReader = new StreamReader(csvFilePath);
                var headline = streamReader.ReadLine();
                while (!streamReader.EndOfStream)
                {
                    var line = streamReader.ReadLine();

                    var parts = line.Split(',');
                    wellDataPoints.Add(new PointShape(double.Parse(parts[1]), double.Parse(parts[2])), double.Parse(parts[3]));
                }
            }
            finally
            {
                streamReader?.Close();
            }
            return wellDataPoints;
        }

        private void CreateGridCellMatrix()
        {
            //Get the current extent since we use that to gerenate the grid.  Of course this is just for
            //the demo and in the real world you can use any extent you like
            RectangleShape currentDrawingExtent = _extent;

            //Calculate the cell size based on how many rows and columns you specified
            double cellSize = 100;
            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, cellSize, -9999, _wellDepthPointData);

            //Generate the grid based on Inverse Distance Weighted interpolation model.  You can define your own model if needed.
            _gridCellMatrix = GridFeatureSource.GenerateGridMatrix(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(5, cellSize * 4));
        }

        
        private InMemoryGridIsoLineLayer GetGridIsoLineLayer()
        {
            //Create a grid cell matrix based on the textboxes and the current extent
            CreateGridCellMatrix();

            //Load the grid we just created into the GridIsoLineLayer using the number of breaks defines
            //on the screen

            var isoLineLayer = new InMemoryGridIsoLineLayer(_gridCellMatrix, _isoLineLevels)
            {
                IsoLineType = rdoLinesOnlyType.IsChecked == true
                    ? IsoLineType.LinesOnly
                    : IsoLineType.ClosedLinesAsPolygons
            };
           

            //Setup a class break style based on the isoline levels and the colors
            var classBreakStyle = new ClassBreakStyle(isoLineLayer.DataValueColumnName);

            var firstStyles = new Collection<ThinkGeo.Core.Style>
            {
                new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[0]), 3)),
                new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                    new GeoSolidBrush(GeoColor.FromHtml(_colorStrings[0])))
            };
            classBreakStyle.ClassBreaks.Add(new ClassBreak(double.MinValue, firstStyles));
            for (var i = 0; i < _isoLineLevels.Count - 1  ; i++)
            {
                var styles = new Collection<ThinkGeo.Core.Style>
                {
                    new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[i + 1]), 3)),
                    new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                        new GeoSolidBrush(GeoColor.FromHtml(_colorStrings[i + 1])))
                };
                classBreakStyle.ClassBreaks.Add(new ClassBreak(_isoLineLevels[i], styles));
            }
            isoLineLayer.CustomStyles.Add(classBreakStyle);

            return isoLineLayer;
        }

        

        #region Control Events

        private void btnGenerateGridFile_Click(object sender, RoutedEventArgs e)
        {
            rdoClosedLinesAsPolygonsType.IsEnabled = true;
            btnGenerateGridFile.IsEnabled = true;

            //Close the previous isoLineLayer and get ready to replace it with a new one
            var layerOverlay = (LayerOverlay)Map1.Overlays["isoLineOverlay"];
            layerOverlay.Layers["IsoLineLayer"].Close();
            layerOverlay.Layers["IsoLineLayer"] = GetGridIsoLineLayer();

            layerOverlay.Refresh();
        }

        private void isoLineOverlay_Drawing(object sender, DrawingOverlayEventArgs e)
        {
            _stopwatch.Reset();
            _stopwatch.Start();

            if (((LayerOverlay)Map1.Overlays["isoLineOverlay"]).Layers["IsoLineLayer"] is IsoLineLayer)
            {
                btnGenerateGridFile.IsEnabled = false;
            }
        }

        private void isoLineOverlay_Drawn(object sender, DrawnOverlayEventArgs e)
        {
            _stopwatch.Stop();
            txtElapseTime.Text = "ElapseTime: " + _stopwatch.ElapsedMilliseconds + "ms";

            if (((LayerOverlay)Map1.Overlays["isoLineOverlay"]).Layers["IsoLineLayer"] is IsoLineLayer)
            {
                btnGenerateGridFile.IsEnabled = true;
            }
        }

        #endregion Control Events
    }
}